package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc;

import java.util.Vector;

/**
 * This class represents EPC AND connectors and is based on EPCConnector.
 * 
 * @author thomas
 */
public class EPCAndConnector extends EPCConnector {

	/**
	 * Constructs a new AND conector.
	 */
	public EPCAndConnector(EPC epc) {
		super(epc);
	}

	/**
	 * An and connector can fire if every edge in preset is marked.
	 */
	@Override
	public boolean canFire() {
		for (EPCEdge edge : getPreset()) {
			if (!edge.isMarked()) {
				return false;
			}
		}

		return true;
	}

	/**
	 * An and connector consumes all marks in preset and produce marks on all
	 * edges in postset.
	 * 
	 * @throws Exception
	 */
	@Override
	public void fire(EPCNode choice) throws Exception {

		Vector<EPCFunction> sources = new Vector<EPCFunction>();

		for (EPCEdge edge : getPreset()) {
			if (edge.isMarked()) {
				sources.addAll(edge.getMarkSources());
			}
			
			edge.setMarked(false, null);
		}

		for (EPCEdge edge : getPostset()) {
			edge.setMarked(true, sources);
		}
	}

	@Override
	public String getLabel() {
		return "and";
	}

	@Override
	public String toString() {
		return "and (" + getPreset().size() + "," + getPostset().size() + ") "
				+ hashCode();
	}
}
