package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc;

import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class represents an EPC function.
 * 
 * @author thomas
 * 
 */
public class EPCFunction extends EPCNode {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory
			.getLogger(EPCFunction.class);

	/**
	 * label of this function
	 */
	private String label;

	/**
	 * Constructs a new EPC function.
	 * 
	 * @param label
	 *            label of this function
	 */
	public EPCFunction(String label, EPC epc) {
		super(epc);
		this.label = label;
	}

	/**
	 * Returns the label of this function.
	 * 
	 * @return label of this function
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * A function can fire if every edge in preset is marked.
	 */
	@Override
	public boolean canFire() {
		for (EPCEdge edge : getPreset()) {
			if (!edge.isMarked()) {
				return false;
			}
		}

		return true;
	}

	/**
	 * A function consumes all marks in preset and produce marks on all edges in
	 * postset.
	 * @throws Exception 
	 */
	@Override
	public void fire(EPCNode choice) throws Exception {
		for (EPCEdge edge : getPreset()) {
			edge.setMarked(false, null);
		}

		for (EPCEdge edge : getPostset()) {
			Vector<EPCFunction> sources = new Vector<EPCFunction>();
			sources.add(this);

			edge.setMarked(true, sources);
		}
	}

	@Override
	public String toString() {
		return getLabel();
	}

	@Override
	public void addIncomingEdge(EPCEdge edge) {
		if (!getPreset().isEmpty()) {
			LOG.error(this + ": A function can only have one incoming edge!");
			try {
				throw new Exception(this
						+ ": A function can only have one incoming edge!");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		super.addIncomingEdge(edge);
	}
}
