package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc;

import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class represents EPC XOR connectors and is based on EPCConnector.
 * 
 * @author thomas
 */
public class EPCXorConnector extends EPCConnector {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory
			.getLogger(EPCXorConnector.class);

	/**
	 * Constructs a new XOR connector.
	 */
	public EPCXorConnector(EPC epc) {
		super(epc);
	}

	/**
	 * A xor connector can fire if one edge in preset is marked.
	 */
	@Override
	public boolean canFire() {
		for (EPCEdge edge : getPreset()) {
			if (edge.isMarked()) {
				return true;
			}
		}

		return false;
	}

	/**
	 * A xor connector consumes all marks in preset and produce marks one edge
	 * in postset.
	 * 
	 * @throws Exception
	 */
	@Override
	public void fire(EPCNode choice) throws Exception {
		EPCEdge choosenEdge = null;

		if (getPostset().size() > 1) {
			if (choice == null) {
				LOG.error(this
						+ ": Choice can not be null if XOR have more outgoing options!");
				try {
					throw new Exception(
							this
									+ ": Choice can not be null if XOR have more outgoing options!");
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			// search for choosen edge
			for (EPCEdge edge : getPostset()) {
				if (edge.getEnd().equals(choice)) {
					choosenEdge = edge;
				}
			}
		} else if (getPostset().size() == 1) {
			choosenEdge = getPostset().firstElement();
		} else {
			LOG.warn("The postset of " + this + " is empty!");
		}

		// get sources of marks in preset
		Vector<EPCFunction> sources = new Vector<EPCFunction>();
		for (EPCEdge edge : getPreset()) {
			if (edge.isMarked()) {
				sources.addAll(edge.getMarkSources());
			}
		}

		if (choosenEdge != null) {
			for (EPCEdge edge : getPreset()) {
				edge.setMarked(false, null);
			}

			choosenEdge.setMarked(true, sources);
		} else {

			LOG.error("The choice " + choice
					+ " is not in postset of the node " + this + " !");
			try {
				throw new Exception("The choice " + choice
						+ " is not in postset of the node " + this + " !");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public String getLabel() {
		return "xor";
	}

	@Override
	public String toString() {
		return "xor (" + getPreset().size() + "," + getPostset().size() + ") "
				+ hashCode();
	}
}
