package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;

public class LooseEnd extends EPCNode {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory.getLogger(LooseEnd.class);

	private Activity start;

	private Activity end;

	private boolean out = true;

	public LooseEnd(EPC epc, Activity from, Activity to, boolean outgoing) {
		super(epc);

		start = from;
		end = to;

		this.out = outgoing;
	}

	public boolean isOutgoingLooseEnd() {
		return out;
	}

	public Activity getStart() {
		return start;
	}

	public Activity getEnd() {
		return end;
	}

	@Override
	public boolean canFire() {
		return false;
	}

	@Override
	public void fire(EPCNode choice) {
		LOG.error("A loose end can not fire!");
	}

	public boolean matches(LooseEnd other) {
		if (out != other.isOutgoingLooseEnd()) {
			if (other.getStart().equals(this.getEnd())) {
				if (other.getEnd().equals(this.getStart())) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public boolean equals(Object other) {
		if (other instanceof LooseEnd) {
			if (out == ((LooseEnd) other).isOutgoingLooseEnd()) {
				if (getStart().equals(((LooseEnd) other).getStart())) {
					if (getEnd().equals(((LooseEnd) other).getEnd())) {
						return true;
					}
				}
			}
		}
		return false;
	}

	@Override
	public String getLabel() {
		String label = "incoming ";
		if (out) {
			label = "outgoing ";
		}
		label += "loose end from " + start.getLabel() + " to " + end.getLabel();
		return label;
	}

	@Override
	public String toString() {
		return getLabel();
	}

	public boolean isValid() {
		if (getPreset().size() == 1 && getPostset().isEmpty()) {
			LOG.debug(this + " is a valid outgoing loose end");
			return true;
		}

		if (getPostset().size() == 1 && getPreset().isEmpty()) {
			LOG.debug(this + " is a valid incoming loose end");
			return true;
		}

		LOG.error(this + " is invalid! preset: " + getPreset().size()
				+ ", postset: " + getPostset().size());
		return false;
	}
}
