package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCNode;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;

public class BlockConnector extends EPCNode {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory
			.getLogger(BlockConnector.class);

	private Activity targetActivity;

	/**
	 * Constructs a new block connector.
	 * 
	 * @param epc
	 *            building block where this connector belongs to
	 * @param target
	 *            target block of this connector as {@link Activity}
	 */
	public BlockConnector(BuildingBlock epc, Activity target) {
		super(epc);

		this.targetActivity = target;
	}

	/**
	 * This method returns the target activity of this block. This is the
	 * activity which is represented by the block where this connector goes to.
	 * 
	 * @return end of this connection as {@link Activity}
	 */
	public Activity getTarget() {
		return targetActivity;
	}

	/**
	 * block connectors can not fire
	 */
	@Override
	public boolean canFire() {
		return false;
	}

	/**
	 * Block connectors can't fire so only a error is fired.
	 */
	@Override
	public void fire(EPCNode choice) {
		LOG.error("A block connector can't fire!");
	}

	/**
	 * Two block connectors are equal if their start and end nodes are equal.
	 */
	@Override
	public boolean equals(Object other) {
		if (other instanceof BlockConnector) {
			Activity blockActivityStart = ((BuildingBlock) this.getEpc())
					.getActivity();
			Activity otherActivityStart = ((BuildingBlock) ((BlockConnector) other)
					.getEpc()).getActivity();
			if (blockActivityStart.equals(otherActivityStart)) {
				if (getTarget().equals(((BlockConnector) other).getTarget())) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public String getLabel() {
		return "Block connector from "
				+ ((BuildingBlock) getEpc()).getActivity().getLabel() + " to "
				+ getTarget();
	}

	@Override
	public String toString() {
		return getLabel();
	}
}
