package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo;

import java.util.HashMap;
import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * An activity is a named event which can occur multiple times in an LPO. The
 * occurences of this activity are represented with vertices.
 * 
 * @author thomas
 * 
 */
public class Activity {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory.getLogger(Activity.class);

	/**
	 * registry for all activities
	 */
	private static HashMap<String, Activity> allActivities = null;

	/**
	 * This method returns the activity with the given label. If the activity
	 * doesn't exist it is created.
	 * 
	 * @param label
	 *            label of the {@link Activity} as {@link String}
	 * @return {@link Activity} with the given label
	 */
	public static Activity getActivity(String label) {
		if (allActivities == null) {
			// create map if not exists
			allActivities = new HashMap<String, Activity>();
		}

		if (!allActivities.containsKey(label)) {
			// create activity if not exists
			allActivities.put(label, new Activity(label));
			LOG.debug("New activity " + label);
		}

		return allActivities.get(label);
	}

	/**
	 * label of this activity
	 */
	private String label;

	/**
	 * list of vertices which represents this activity
	 */
	private Vector<Vertex> vertices;

	/**
	 * Constructs a new {@link Activity} with the given label.
	 * 
	 * @param label
	 *            label as {@link String}
	 */
	private Activity(String label) {
		this.label = label;

		this.vertices = new Vector<Vertex>();
	}

	public Vertex getVertex() {
		return new Vertex(this);
	}

	public Vector<Vertex> getAllVertices() {
		return vertices;
	}

	public void addVertex(Vertex vertex) {
		if (!this.equals(vertex.getActivity())) {
			LOG.error("Vertex was not added! " + getLabel()
					+ " is not equal with " + vertex.getActivity().getLabel());
			return;
		}

		if (!vertices.contains(vertex)) {
			vertices.add(vertex);
		}
	}

	/**
	 * This method returns the label of this activity.
	 * 
	 * @return label as {@link String}
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * Should return the label of the activity. Is used by equals.
	 */
	@Override
	public String toString() {
		return label;
	}

	/**
	 * Two activities are equal if their labels are equal.
	 */
	@Override
	public boolean equals(Object o) {
		if (o instanceof Activity) {
			if (label.equals(o.toString())) {
				return true;
			}
		}
		return false;
	}
}
