package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo;

/**
 * This class represents an edge of a lpo.
 * 
 * @author thomas
 * 
 */
public class LpoEdge {

	/**
	 * vertex where the edge starts
	 */
	private Vertex start;

	/**
	 * vertex where the edge ends
	 */
	private Vertex end;

	/**
	 * true if part of the skeletton
	 */
	private boolean skeletonEdge;

	/**
	 * true if edge is part of the lpo defintion
	 */
	private boolean modeledEdge;

	/**
	 * true if used for folding
	 */
	private boolean foldingEdge;

	/**
	 * Constructs a new edge form start to end
	 * 
	 * @param start
	 *            start vertex of the edge
	 * @param end
	 *            end vertex of the edge
	 */
	public LpoEdge(Vertex start, Vertex end) {
		this(start, end, false);
	}

	/**
	 * Constructs a new edge form start to end.
	 * 
	 * @param start
	 *            start vertex of the edge
	 * @param end
	 *            end vertex of the edge
	 * @param modelled
	 *            modelled by user
	 */
	public LpoEdge(Vertex start, Vertex end, boolean modelled) {
		this.start = start;
		this.end = end;

		skeletonEdge = false;
		modeledEdge = modelled;
		foldingEdge = false;
	}

	/**
	 * This method returns true if it is an skeleton edge.
	 * 
	 * @return true of skeleton
	 */
	public boolean isSkeletonEdge() {
		return skeletonEdge;
	}

	/**
	 * This method sets the skeleton state.
	 * 
	 * @param skeletonEdge
	 *            true if skeleton edge
	 */
	public void setSkeletonEdge(boolean skeletonEdge) {
		this.skeletonEdge = skeletonEdge;
	}

	/**
	 * This method returns true if the edge was modelled by user.
	 * 
	 * @return true if modelled by user
	 */
	public boolean isModeledEdge() {
		return modeledEdge;
	}

	/**
	 * This method sets if the edge was modeled by user.
	 * 
	 * @param modelledEdge
	 *            true if user modeled
	 */
	public void setModeledEdge(boolean modeledEdge) {
		this.modeledEdge = modeledEdge;
	}

	/**
	 * This method returns if the edge is used for folding.
	 * 
	 * @return true if
	 */
	public boolean isFoldingEdge() {
		return foldingEdge;
	}

	/**
	 * This method sets if this edge is used for folding.
	 * 
	 * @param foldingEdge
	 *            true if used for folding
	 */
	public void setFoldingEdge(boolean foldingEdge) {
		this.foldingEdge = foldingEdge;
	}

	/**
	 * Returns the start vertex of this edge.
	 * 
	 * @return start vertex
	 */
	public Vertex getStart() {
		return start;
	}

	/**
	 * Returns the end vertex of this edge.
	 * 
	 * @return end vertex of the edge
	 */
	public Vertex getEnd() {
		return end;
	}

	@Override
	public boolean equals(Object other) {
		if (other instanceof LpoEdge) {
			LpoEdge otherEdge = (LpoEdge) other;
			if (getStart().equals(otherEdge.getStart())) {
				if (getEnd().equals(otherEdge.getEnd())) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public String toString() {
		return "LpoEdge from " + start.getActivity().getLabel() + " to "
				+ end.getActivity().getLabel();
	}
}
