package de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.exec;

import java.util.HashMap;
import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.LpoEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Vertex;

public class LpoState {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory.getLogger(LpoState.class);

	private Lpo lpo;

	private HashMap<Activity, Vector<Vertex>> enabled;

	public LpoState(Lpo lpo) {
		this.lpo = lpo;

		LOG.debug("New LpoState for " + lpo.getName());

		enabled = new HashMap<Activity, Vector<Vertex>>();

		enabled.put(lpo.getStartVertex().getActivity(), new Vector<Vertex>());
		enabled.get(lpo.getStartVertex().getActivity()).add(
				lpo.getStartVertex());
	}

	public boolean canFire(String label) {
		return canFire(Activity.getActivity(label));
	}

	public boolean canFire(Activity activity) {
		if (enabled.containsKey(activity)) {
			if (!enabled.get(activity).isEmpty()) {
				return true;
			}
		}
		return false;
	}

	public Vector<Vertex> getEnabledVertices(String label) {
		return getEnabledVertices(Activity.getActivity(label));
	}

	public Vector<Vertex> getEnabledVertices(Activity activity) {
		if (enabled.containsKey(activity)) {
			return enabled.get(activity);
		}

		return new Vector<Vertex>();
	}

	public void fire(Vertex vertex) {
		enabled.get(vertex.getActivity()).remove(vertex);

		for (LpoEdge edge : vertex.getSkeletonPostset()) {
			if (!enabled.containsKey(edge.getEnd().getActivity())) {
				enabled.put(edge.getEnd().getActivity(), new Vector<Vertex>());
			}

			enabled.get(edge.getEnd().getActivity()).add(edge.getEnd());
		}

		LOG.debug(this + ": Fire " + vertex.getActivity().getLabel()
				+ ". Enabled " + enabled);
	}

	public Lpo getLpo() {
		return lpo;
	}

	public LpoState clone() {
		LpoState clone = new LpoState(lpo);

		for (Activity key : enabled.keySet()) {
			Vector<Vertex> vertices = this.enabled.get(key);

			Vector<Vertex> verticesClone = new Vector<Vertex>();
			verticesClone.addAll(vertices);
			clone.enabled.put(key, verticesClone);
		}

		return clone;
	}

	@Override
	public String toString() {
		return lpo.getName() + " (" + enabled + ")";
	}
}
