package de.fernunihagen.viptool.algorithm.folding.epcoflpo.io.lpo;

import java.io.File;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import viptool.base.error.XMLException;
import viptool.util.XMLUtils;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Vertex;

public class LpoParser {

	public static void main(String[] args) {
		LpoParser parser = new LpoParser();

		parser.parseLpo(new File("lpoUV.lpo"));
	}

	private Lpo currentLpo = null;

	private HashMap<String, Vertex> events = null;

	public LpoParser() {

	}

	public Lpo parseLpo(File file) {

		currentLpo = new Lpo(file.getName());
		events = new HashMap<String, Vertex>();

		try {

			Document doc = XMLUtils.parseXMLFile(file.toURI());

			NodeList nl = doc.getElementsByTagName("lpo");
			if (nl.getLength() >= 1) {
				Element lpo = (Element) nl.item(0);
				load(lpo);
			}

		} catch (XMLException e) {
			e.printStackTrace();
		}

		currentLpo.finishConstruction();
		
		return currentLpo;
	}

	private void load(Element lpoElement) {
		if (!(lpoElement.getNodeName().equals("lpo"))) {
			System.err.println("File to load is no lpo file!");
		} else {
			NodeList childNodes = lpoElement.getChildNodes();

			for (int i = 0; i < childNodes.getLength(); i++) {
				if (childNodes.item(i).getNodeName().equals("event")) {
					addNode((Element) childNodes.item(i));
				}
			}

			for (int i = 0; i < childNodes.getLength(); i++) {
				if (childNodes.item(i).getNodeName().equals("lpoArc")) {
					addArc((Element) childNodes.item(i));
				}
			}

		}
	}

	private void addNode(Element node) {

		String id = node.getAttribute("id");

		NodeList nl = node.getElementsByTagName("name");
		Element name = (Element) nl.item(0);
		nl = name.getElementsByTagName("value");
		String label = nl.item(0).getTextContent();

		Vertex v = currentLpo.addVertex(label);
		events.put(id, v);
	}

	private void addArc(Element arc) {

		String source = arc.getAttribute("source");
		String target = arc.getAttribute("target");

		Vertex sv = events.get(source);
		Vertex tv = events.get(target);

		currentLpo.connectVertices(sv, tv);
	}
}
