package de.fernunihagen.viptool.algorithm.folding.epcoflpo.plugin;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import viptool.base.IWorkbench;
import viptool.base.action.AbstractFileSelectionAction;
import viptool.base.gui.selection.IAbstractFileSelection;
import viptool.base.io.IFileInterpreter;
import viptool.base.wizard.IWizard;
import viptool.base.wizard.NewFileWizard;
import viptool.epc.io.EpcFileProvider;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IProject;
import viptool.util.GeneralUtils;

public class EpcFoldingAction extends AbstractFileSelectionAction {

	private static final long serialVersionUID = 1L;

	public void actionPerformed(ActionEvent event,
			IAbstractFileSelection selection) {

		int index;
		Iterator it;
		String[] files;
		IFileInterpreter epcFile;
		IProject project;
		IAbstractFile file;
		IWizard wizard;
		IWorkbench workbench;
		IAbstractFolder[] folders;
		IAbstractFile[] abstractFiles;

		super.actionPerformed(event, selection);

		// get the filenames
		index = 0;
		project = null;

		folders = selection.getFolderElements();
		if (folders.length > 0) {
			files = new String[folders[0].getSize()];
			abstractFiles = new IAbstractFile[folders[0].getSize()];
			it = folders[0].getChildIterator();
		} else {
			files = new String[selection.size()];
			abstractFiles = new IAbstractFile[selection.size()];
			it = selection.iterator();
		}

		while (it.hasNext()) {
			file = (IAbstractFile) it.next();
			abstractFiles[index] = file;
			project = file.getProject();

			files[index++] = file.getFile().toURI().toString();
		}

		// test if the files were modified
		if (!GeneralUtils
				.testForFileLocker(
						EpcFoldingExtension.getInstance().getWorkbench(),
						abstractFiles,
						"The file '{file}' has been modified. Click 'OK' to save the file first, or 'Cancel' to abort the algorithm.")) {
			// aborted
			return;
		}

		// get the petrinet output file from the user
		workbench = EpcFoldingExtension.getInstance().getWorkbench();

		wizard = workbench.getFileInterpreterManager()
				.getFileInterpreterProvider(EpcFileProvider.class)
				.getNewFileWizard();

		workbench.startWizard(wizard);

		epcFile = ((NewFileWizard) wizard).getCreatedFile();
		if (epcFile == null) {
			return;
		}

		EpcFoldingExtension.getInstance().getWorkbench()
				.closeAllEditors(epcFile.getAbstractFile(), true);

		EpcFoldingExtension
				.getInstance()
				.getWorkbench()
				.getJobScheduler()
				.addJob(new EpcFoldingJob(project, files, epcFile
						.getAbstractFile().getFullPath()));
	}

}
