package de.fernunihagen.viptool.algorithm.folding.epcoflpo.run;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.Level;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks.BuildingBlock;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.exec.WrongContinuation;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.util.ActivityPair;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.io.epml.EPMLDocumentBuilder;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.io.lpo.LpoParser;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.logic.EpcFolding;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.logic.FindPossibleEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.logic.FindWrongContinuation;

public class FoldLposToEpc {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory
			.getLogger(FoldLposToEpc.class);

	public static void prepareLogger() {
		final ch.qos.logback.classic.Logger root = (ch.qos.logback.classic.Logger) org.slf4j.LoggerFactory
				.getLogger(Logger.ROOT_LOGGER_NAME);
		root.setLevel(Level.INFO);

	}

	public static void prepareLogger(Level level) {
		final ch.qos.logback.classic.Logger root = (ch.qos.logback.classic.Logger) org.slf4j.LoggerFactory
				.getLogger(Logger.ROOT_LOGGER_NAME);
		root.setLevel(level);

	}

	public static void main(String[] args) {
		foldLposToEpc(args, "foldedModel.epc");
	}

	public static void foldLposToEpc(String[] input, String output) {

		FoldLposToEpc.prepareLogger();

		EpcFolding algo = new EpcFolding();

		Vector<Lpo> specification = new Vector<Lpo>();

		LpoParser parser = new LpoParser();

		for (String filePath : input) {
			if (filePath != null) {
				File file = null;

				if (filePath.startsWith("file:")) {
					try {
						file = new File(new URI(filePath));
					} catch (URISyntaxException e) {
						e.printStackTrace();
						continue;
					}
				} else {
					file = new File(filePath);
				}

				if (file.exists()) {
					try {
						specification.add(parser.parseLpo(file));
					} catch (Exception e) {
						LOG.error("Input file " + filePath
								+ " couln't be parsed!");
					}
				} else {
					LOG.warn("Input file " + filePath + " doesn't exist!");
				}
			} else {
				LOG.warn("Input file was null!");
			}
		}

		// add the lpos of the paper
		algo.addToSpecification(specification);

		boolean fixed = false;
		boolean finished = false;

		do {
			// calculate the building blocks
			HashMap<Activity, BuildingBlock> blocks = algo
					.generateBuildingBlocks(algo.getSpecification());
			// merge the building blocks
			EPC epc = algo.mergeBlocks(blocks);

			EPMLDocumentBuilder.saveEpcAsFile(epc, new File(output));

			prepareLogger(Level.INFO);

			FindWrongContinuation wcAlgo = new FindWrongContinuation();
			wcAlgo.addToSpecification(specification);
			WrongContinuation wc = wcAlgo.runAlgo(epc);

			LOG.info("Wrong Continuation: " + wc);

			if (wc != null) {
				prepareLogger(Level.INFO);

				FindPossibleEdge findEdgeAlgo = new FindPossibleEdge(algo, wc);
				ActivityPair pair = findEdgeAlgo.runAlgo();
				if (pair == null) {
					LOG.warn("The wrong continuation " + wc
							+ " couldn't be fixed!");
					fixed = false;
				} else {
					fixed = true;
				}
			} else {
				LOG.info("Algorithm finished. Result written to " + output);
				finished = true;
			}
		} while (fixed && !finished);
	}
}
