package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.epc;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCAndConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCFunction;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCNode;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCXorConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.LooseEnd;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;

public class TestEPC {

	@Before
	public void setUp() throws Exception {

	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testAddLooseEnd() {
		EPC epc = new EPC();

		LooseEnd loose1 = epc.addLooseEnd(Activity.getActivity("Start"),
				Activity.getActivity("End"), true);
		LooseEnd loose2 = epc.addLooseEnd(Activity.getActivity("End"),
				Activity.getActivity("Start"), false);

		assertTrue(epc.getLooseEnds().size() == 2);
		assertTrue(epc.getLooseEnds().contains(loose1));
		assertTrue(epc.getLooseEnds().contains(loose2));

	}

	@Test
	public void testRemoveAllLoseEnds() {
		EPC epc = new EPC();

		LooseEnd loose1 = epc.addLooseEnd(Activity.getActivity("Start"),
				Activity.getActivity("End"), true);
		LooseEnd loose2 = epc.addLooseEnd(Activity.getActivity("End"),
				Activity.getActivity("Start"), false);

		assertTrue(epc.getLooseEnds().size() == 2);
		assertTrue(epc.getLooseEnds().contains(loose1));
		assertTrue(epc.getLooseEnds().contains(loose2));

		epc.removeAllLoseEnds();

		assertTrue(epc.getLooseEnds().isEmpty());
	}

	@Test
	public void testAddFunction() {
		EPC epc = new EPC();

		EPCFunction function = epc.addFunction("testFunction");

		assertTrue(epc.getFunctions().size() == 1);
		assertTrue(epc.getFunctions().contains(function));
	}

	@Test
	public void testAddAndConnector() {
		EPC epc = new EPC();

		EPCAndConnector and = epc.addAndConnector();

		assertTrue(epc.getAndConnectors().size() == 1);
		assertTrue(epc.getAndConnectors().contains(and));
	}

	@Test
	public void testAddXorConnector() {
		EPC epc = new EPC();

		EPCXorConnector xor = epc.addXorConnector();

		assertTrue(epc.getXorConnectors().size() == 1);
		assertTrue(epc.getXorConnectors().contains(xor));
	}

	@Test
	public void testRemoveUselessConnectors() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");
		EPCXorConnector xor = epc.addXorConnector();

		epc.connect(function1, xor);
		epc.connect(xor, function2);

		assertTrue(function1.getPostset().firstElement().getEnd().equals(xor));
		assertTrue(function2.getPreset().firstElement().getStart().equals(xor));

		assertTrue(xor.getPreset().firstElement().getStart().equals(function1));
		assertTrue(xor.getPostset().firstElement().getEnd().equals(function2));

		assertTrue(xor.getPreset().size() == 1);
		assertTrue(xor.getPostset().size() == 1);

		assertTrue(epc.getXorConnectors().size() == 1);

		epc.removeUselessConnectors();

		assertTrue(epc.getXorConnectors().isEmpty());
		assertTrue(function1.getPostset().firstElement().getEnd()
				.equals(function2));
		assertTrue(function2.getPreset().firstElement().getStart()
				.equals(function1));
	}

	@Test
	public void testConnect() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");

		epc.connect(function1, function2);

		assertTrue(function1.getPostset().firstElement().getEnd()
				.equals(function2));
		assertTrue(function2.getPreset().firstElement().getStart()
				.equals(function1));
	}

	@Test
	public void testDisconnect() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");

		epc.connect(function1, function2);

		assertTrue(function1.getPostset().firstElement().getEnd()
				.equals(function2));
		assertTrue(function2.getPreset().firstElement().getStart()
				.equals(function1));

		epc.disconnect(function1, function2);

		assertTrue(function1.getPostset().isEmpty());
		assertTrue(function2.getPreset().isEmpty());

	}

	@Test
	public void testGetFunctions() {
		EPC epc = new EPC();

		assertTrue(epc.getFunctions().isEmpty());

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");

		assertTrue(epc.getFunctions().size() == 2);
		assertTrue(epc.getFunctions().contains(function1));
		assertTrue(epc.getFunctions().contains(function2));
	}

	@Test
	public void testGetAndConnectors() {
		EPC epc = new EPC();

		assertTrue(epc.getAndConnectors().isEmpty());

		EPCConnector con1 = epc.addAndConnector();
		EPCConnector con2 = epc.addAndConnector();

		assertTrue(epc.getAndConnectors().size() == 2);
		assertTrue(epc.getAndConnectors().contains(con1));
		assertTrue(epc.getAndConnectors().contains(con2));
	}

	@Test
	public void testGetXorConnectors() {
		EPC epc = new EPC();

		assertTrue(epc.getXorConnectors().isEmpty());

		EPCConnector con1 = epc.addXorConnector();
		EPCConnector con2 = epc.addXorConnector();

		assertTrue(epc.getXorConnectors().size() == 2);
		assertTrue(epc.getXorConnectors().contains(con1));
		assertTrue(epc.getXorConnectors().contains(con2));
	}

	@Test
	public void testCanFire() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");

		assertTrue(function1.canFire());
		assertTrue(function2.canFire());

		Vector<EPCNode> nodes = new Vector<EPCNode>();
		nodes.add(function1);
		nodes.add(function2);

		assertTrue(epc.canFire(nodes));

		epc.connect(function1, function2);
		try {
			function1.getPostset().firstElement().setMarked(false, null);
		} catch (Exception e) {
			e.printStackTrace();
		}

		assertFalse(function2.canFire());
		assertFalse(epc.canFire(nodes));
	}

	@Test
	public void testMarkWithInitialMarking() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");
		EPCFunction function3 = epc.addFunction("function3");

		epc.connect(function1, function2);
		epc.connect(function2, function3);

		epc.markWithInitialMarking();

		assertTrue(function1.canFire());
		assertTrue(function2.canFire());
		assertFalse(function3.canFire());
	}

	@Test
	public void testGetInitialNodes() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");
		EPCFunction function3 = epc.addFunction("function3");

		epc.connect(function1, function2);
		epc.connect(function2, function3);

		assertTrue(epc.getInitialNodes().size() == 1);
		assertTrue(epc.getInitialNodes().contains(function1));
	}

	@Test
	public void testGetAllEdges() {
		EPC epc = new EPC();

		EPCFunction function1 = epc.addFunction("function1");
		EPCFunction function2 = epc.addFunction("function2");
		EPCFunction function3 = epc.addFunction("function3");

		epc.connect(function1, function2);
		epc.connect(function2, function3);

		assertTrue(epc.getAllEdges().size() == 2);

		assertTrue(epc.getAllEdges().containsAll(function1.getPreset()));
		assertTrue(epc.getAllEdges().containsAll(function1.getPostset()));

		assertTrue(epc.getAllEdges().containsAll(function2.getPreset()));
		assertTrue(epc.getAllEdges().containsAll(function2.getPostset()));

		assertTrue(epc.getAllEdges().containsAll(function3.getPreset()));
		assertTrue(epc.getAllEdges().containsAll(function3.getPostset()));
	}
}
