package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.epc;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCAndConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCFunction;

public class TestEPCAndConnector {

	private EPC epc;
	private EPCAndConnector pre1;
	private EPCAndConnector pre2;
	private EPCAndConnector post1;
	private EPCAndConnector post2;

	@Before
	public void setUp() throws Exception {
		epc = new EPC();

		pre1 = epc.addAndConnector();
		pre2 = epc.addAndConnector();
		post1 = epc.addAndConnector();
		post2 = epc.addAndConnector();

	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testCanFire() {
		EPCAndConnector and = epc.addAndConnector();

		epc.connect(pre1, and);
		epc.connect(pre2, and);

		for (EPCEdge edge : and.getPreset()) {
			try {
				edge.setMarked(false, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		assertFalse(and.canFire());

		try {
			Vector<EPCFunction> markSources = new Vector<EPCFunction>();
			markSources.add(epc.addFunction("TestFunction"));
			and.getPreset().firstElement().setMarked(true, markSources);
		} catch (Exception e) {
			e.printStackTrace();
		}

		assertFalse(and.canFire());

		for (EPCEdge edge : and.getPreset()) {
			try {
				Vector<EPCFunction> markSources = new Vector<EPCFunction>();
				markSources.add(epc.addFunction("TestFunction"));
				edge.setMarked(true, markSources);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		assertTrue(and.canFire());
	}

	@Test
	public void testFire() {
		EPCAndConnector and = epc.addAndConnector();

		epc.connect(pre1, and);
		epc.connect(pre2, and);
		epc.connect(and, post1);
		epc.connect(and, post2);

		// TODO: test markSources

		for (EPCEdge edge : and.getPreset()) {
			try {
				Vector<EPCFunction> markSources = new Vector<EPCFunction>();
				markSources.add(epc.addFunction("TestFunction"));
				edge.setMarked(true, markSources);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		for (EPCEdge edge : and.getPostset()) {
			try {
				edge.setMarked(false, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		try {
			and.fire(null);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for (EPCEdge edge : and.getPostset()) {
			assertTrue(edge.isMarked());
		}

		for (EPCEdge edge : and.getPreset()) {
			assertFalse(edge.isMarked());
		}
	}

}
