package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.epc;

import static org.junit.Assert.assertTrue;

import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCAndConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCFunction;

public class TestEPCEdge {

	private EPC epc;
	private EPCAndConnector pre1;
	private EPCAndConnector pre2;
	private EPCAndConnector post1;
	private EPCAndConnector post2;

	@Before
	public void setUp() throws Exception {
		epc = new EPC();

		pre1 = epc.addAndConnector();
		pre2 = epc.addAndConnector();
		post1 = epc.addAndConnector();
		post2 = epc.addAndConnector();

	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testGetEdge() {
		EPCEdge edge = EPCEdge.getEdge(pre1, post1, epc);

		assertTrue(edge.getEpc().equals(epc));
		assertTrue(edge.getStart().equals(pre1));
		assertTrue(edge.getEnd().equals(post1));
	}

	@Test
	public void testGetAllEdges() {
		int count = EPCEdge.getAllEdges(epc).size();

		EPCEdge edge = EPCEdge.getEdge(pre1, pre2, epc);

		assertTrue(EPCEdge.getAllEdges(epc).contains(edge));
		assertTrue(EPCEdge.getAllEdges(epc).size() == (count + 1));
	}

	@Test
	public void testDeleteEdge() {
		int count = EPCEdge.getAllEdges(epc).size();

		EPCEdge edge = EPCEdge.getEdge(pre2, pre1, epc);

		assertTrue(EPCEdge.getAllEdges(epc).contains(edge));
		assertTrue(EPCEdge.getAllEdges(epc).size() == (count + 1));

		EPCEdge.deleteEdge(pre2, pre1);

		assertTrue(!EPCEdge.getAllEdges(epc).contains(edge));
		assertTrue(EPCEdge.getAllEdges(epc).size() == count);
	}

	@Test
	public void testGetEpc() {
		EPCEdge edge = EPCEdge.getEdge(pre2, post2, epc);

		assertTrue(edge.getEpc().equals(epc));
	}

	@Test
	public void testGetStart() {
		EPCEdge edge = EPCEdge.getEdge(pre2, post2, epc);

		assertTrue(edge.getEpc().equals(epc));
	}

	@Test
	public void testGetEnd() {
		EPCEdge edge = EPCEdge.getEdge(pre2, post2, epc);

		assertTrue(edge.getEnd().equals(post2));
	}

	@Test
	public void testIsMarked() {
		EPCEdge edge = EPCEdge.getEdge(pre2, post2, epc);

		// TODO: test mark sources

		try {
			Vector<EPCFunction> markSources = new Vector<EPCFunction>();
			markSources.add(epc.addFunction("TestFunction"));
			edge.setMarked(true, markSources);
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		assertTrue(edge.isMarked());

		try {
			edge.setMarked(false, null);
		} catch (Exception e) {
			e.printStackTrace();
		}
		assertTrue(!edge.isMarked());
	}

	@Test
	public void testEqualsObject() {
		EPCEdge edge1 = EPCEdge.getEdge(pre2, post2, epc);
		EPCEdge edge2 = EPCEdge.getEdge(pre2, post2, epc);

		assertTrue(edge1.equals(edge2));
	}

}
