package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.epc;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCAndConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCFunction;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCXorConnector;

public class TestEPCXorConnector {

	private EPC epc;
	private EPCAndConnector pre1;
	private EPCAndConnector pre2;
	private EPCAndConnector post1;
	private EPCAndConnector post2;

	@Before
	public void setUp() throws Exception {
		epc = new EPC();

		pre1 = epc.addAndConnector();
		pre2 = epc.addAndConnector();
		post1 = epc.addAndConnector();
		post2 = epc.addAndConnector();

	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testCanFire() {
		EPCXorConnector xor = epc.addXorConnector();

		epc.connect(pre1, xor);
		epc.connect(pre2, xor);

		// TODO: test mark sources

		for (EPCEdge edge : xor.getPreset()) {
			try {
				edge.setMarked(false, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		assertFalse(xor.canFire());

		try {
			Vector<EPCFunction> markSources = new Vector<EPCFunction>();
			markSources.add(epc.addFunction("TestFunction"));
			xor.getPreset().firstElement().setMarked(true, markSources);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		assertTrue(xor.canFire());
	}

	@Test
	public void testFire() {
		EPCXorConnector xor = epc.addXorConnector();

		epc.connect(pre1, xor);
		epc.connect(pre2, xor);
		epc.connect(xor, post1);
		epc.connect(xor, post2);

		for (EPCEdge edge : xor.getPreset()) {
			try {
				edge.setMarked(false, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		try {
			Vector<EPCFunction> markSources = new Vector<EPCFunction>();
			markSources.add(epc.addFunction("TestFunction"));
			xor.getPreset().firstElement().setMarked(true, markSources);
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			xor.fire(post1);
		} catch (Exception e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		for (EPCEdge edge : xor.getPostset()) {
			if (edge.getEnd().equals(post1)) {
				assertTrue(edge.isMarked());
			} else if (edge.getEnd().equals(post2)) {
				assertFalse(edge.isMarked());
			} else {
				fail("Unexpected edge " + edge + " in postset.");
			}
		}

		for (EPCEdge edge : xor.getPreset()) {
			assertFalse(edge.isMarked());
		}

		for (EPCEdge edge : xor.getPreset()) {
			try {
				edge.setMarked(false, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		for (EPCEdge edge : xor.getPostset()) {
			try {
				edge.setMarked(false, null);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		try {
			Vector<EPCFunction> markSources = new Vector<EPCFunction>();
			markSources.add(epc.addFunction("TestFunction"));
			xor.getPreset().firstElement().setMarked(true, markSources);
		} catch (Exception e) {
			e.printStackTrace();
		}

		try {
			xor.fire(post2);
		} catch (Exception e) {
			e.printStackTrace();
		}

		for (EPCEdge edge : xor.getPostset()) {
			if (edge.getEnd().equals(post1)) {
				assertFalse(edge.isMarked());
			} else if (edge.getEnd().equals(post2)) {
				assertTrue(edge.isMarked());
			} else {
				fail("Unexpected edge " + edge + " in postset.");
			}
		}

		for (EPCEdge edge : xor.getPreset()) {
			assertFalse(edge.isMarked());
		}
	}
}
