package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.epc;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.LooseEnd;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;

public class TestLooseEnd {

	private EPC epc;

	@Before
	public void setUp() throws Exception {
		epc = new EPC();
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testCanFire() {

		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), true);
		assertFalse(looseEnd.canFire());
	}

	@Test
	public void testLooseEnd() {
		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), false);

		assertFalse(looseEnd.isOutgoingLooseEnd());
		assertTrue(looseEnd.getStart().equals(Activity.getActivity("Start")));
		assertTrue(looseEnd.getEnd().equals(Activity.getActivity("End")));
		assertTrue(looseEnd.getEpc().equals(epc));
	}

	@Test
	public void testIsOutgoingLooseEnd() {
		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), true);
		assertTrue(looseEnd.isOutgoingLooseEnd());

		looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), false);
		assertFalse(looseEnd.isOutgoingLooseEnd());
	}

	@Test
	public void testGetStart() {
		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), true);

		assertTrue(looseEnd.getStart().equals(Activity.getActivity("Start")));
	}

	@Test
	public void testGetEnd() {
		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), true);

		assertTrue(looseEnd.getEnd().equals(Activity.getActivity("End")));
	}

	@Test
	public void testMatches() {
		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), true);

		LooseEnd otherLooseEnd = new LooseEnd(epc, Activity.getActivity("End"),
				Activity.getActivity("Start"), false);

		assertTrue(looseEnd.matches(otherLooseEnd));
	}

	@Test
	public void testEqualsObject() {
		LooseEnd looseEnd = new LooseEnd(epc, Activity.getActivity("Start"),
				Activity.getActivity("End"), true);

		LooseEnd otherLooseEnd = new LooseEnd(epc,
				Activity.getActivity("Start"), Activity.getActivity("End"),
				true);

		LooseEnd thirdLooseEnd = new LooseEnd(epc,
				Activity.getActivity("Start"), Activity.getActivity("End"),
				false);

		assertTrue(looseEnd.equals(otherLooseEnd));
		assertTrue(!looseEnd.equals(thirdLooseEnd));
	}
}
