package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCAndConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCFunction;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCNode;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCXorConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.LooseEnd;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks.ActivitySet;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks.BlockConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks.BuildingBlock;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;

public class TestBuildingBlock {

	private Activity blockActivity;
	private ActivitySet activityset1;
	private ActivitySet activityset2;

	@Before
	public void setUp() throws Exception {
		blockActivity = Activity.getActivity("BlockActivity");

		activityset1 = new ActivitySet();
		activityset1.add(Activity.getActivity("A"));
		activityset1.add(Activity.getActivity("B"));
		activityset1.add(Activity.getActivity("C"));

		activityset2 = new ActivitySet();
		activityset2.add(Activity.getActivity("C"));
		activityset2.add(Activity.getActivity("D"));
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testBuildingBlock() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		block.getActivity().equals(blockActivity);
	}

	@Test
	public void testAddPreset() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		block.addPreset(activityset1);

		assertTrue(block.getPreConnectors().size() + " != 3", block
				.getPreConnectors().size() == 3);

		Vector<Activity> targets = new Vector<Activity>();
		for (BlockConnector connector : block.getPreConnectors()) {
			targets.add(connector.getTarget());
		}

		assertTrue(targets.contains(Activity.getActivity("A")));
		assertTrue(targets.contains(Activity.getActivity("B")));
		assertTrue(targets.contains(Activity.getActivity("C")));

		assertTrue(block.getFunctions().size() == 1);

		EPCFunction function = block.getFunctions().firstElement();

		assertTrue(function.getPreset().size() == 1);

		EPCNode preXor = function.getPreset().firstElement().getStart();

		assertTrue(preXor instanceof EPCXorConnector);
		assertTrue(preXor.getPreset().size() == 1);

		EPCNode preAnd = preXor.getPreset().firstElement().getStart();

		assertTrue(preAnd instanceof EPCAndConnector);
		assertTrue(preAnd.getPreset().size() == 3);

		for (EPCEdge edge : preAnd.getPreset()) {
			EPCNode node = edge.getStart();
			assertTrue(node instanceof EPCXorConnector);
			assertTrue(node.getPreset().size() == 1);
			assertTrue(node.getPreset().firstElement().getStart() instanceof BlockConnector);
		}

		// add second preset

		block.addPreset(activityset2);

		assertTrue(block.getPreConnectors().size() + " != 4", block
				.getPreConnectors().size() == 4);

		targets.clear();
		for (BlockConnector connector : block.getPreConnectors()) {
			targets.add(connector.getTarget());
		}

		assertTrue(targets.contains(Activity.getActivity("A")));
		assertTrue(targets.contains(Activity.getActivity("B")));
		assertTrue(targets.contains(Activity.getActivity("C")));
		assertTrue(targets.contains(Activity.getActivity("D")));

		assertTrue(function.getPreset().size() == 1);

		assertTrue(preXor.getPreset().size() == 2);

		EPCNode secondAnd = null;
		for (EPCEdge edge : preXor.getPreset()) {
			if (!edge.getStart().equals(preAnd)) {
				secondAnd = edge.getStart();
				break;
			}
		}

		assertTrue(secondAnd instanceof EPCAndConnector);
		assertTrue(secondAnd.getPreset().size() == 2);

		for (EPCEdge edge : preAnd.getPreset()) {
			EPCNode node = edge.getStart();
			assertTrue(node instanceof EPCXorConnector);
			assertTrue(node.getPreset().size() == 1);
			assertTrue(node.getPreset().firstElement().getStart() instanceof BlockConnector);
		}

		assertTrue(block.getPreConnectors().size() == 4);

		for (BlockConnector connector : block.getPreConnectors()) {
			if (connector.getTarget().equals(Activity.getActivity("A"))) {
				assertTrue(connector.getPostset().size() == 1);
			} else if (connector.getTarget().equals(Activity.getActivity("B"))) {
				assertTrue(connector.getPostset().size() == 1);
			} else if (connector.getTarget().equals(Activity.getActivity("C"))) {
				assertTrue(connector.getPostset().size() == 1);
				EPCNode xor = connector.getPostset().firstElement().getEnd();
				assertTrue(xor instanceof EPCXorConnector);
				assertTrue(xor.getPostset().size() == 2);
			} else if (connector.getTarget().equals(Activity.getActivity("D"))) {
				assertTrue(connector.getPostset().size() == 1);
			} else {
				fail("Unexpected connector: " + connector);
			}
		}
	}

	@Test
	public void testAddPostset() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		block.addPostset(activityset1);

		assertTrue(block.getPostConnectors().size() + " != 3", block
				.getPostConnectors().size() == 3);

		Vector<Activity> targets = new Vector<Activity>();
		for (BlockConnector connector : block.getPostConnectors()) {
			targets.add(connector.getTarget());
		}

		assertTrue(targets.contains(Activity.getActivity("A")));
		assertTrue(targets.contains(Activity.getActivity("B")));
		assertTrue(targets.contains(Activity.getActivity("C")));

		assertTrue(block.getFunctions().size() == 1);

		EPCFunction function = block.getFunctions().firstElement();

		assertTrue(function.getPostset().size() == 1);

		EPCNode postXor = function.getPostset().firstElement().getEnd();

		assertTrue(postXor instanceof EPCXorConnector);
		assertTrue(postXor.getPostset().size() == 1);

		EPCNode postAnd = postXor.getPostset().firstElement().getEnd();

		assertTrue(postAnd instanceof EPCAndConnector);
		assertTrue(postAnd.getPostset().size() == 3);

		for (EPCEdge edge : postAnd.getPostset()) {
			EPCNode node = edge.getEnd();
			assertTrue(node instanceof EPCXorConnector);
			assertTrue(node.getPostset().size() == 1);
			assertTrue(node.getPostset().firstElement().getEnd() instanceof BlockConnector);
		}

		// add second preset

		block.addPostset(activityset2);

		assertTrue(block.getPostConnectors().size() + " != 4", block
				.getPostConnectors().size() == 4);

		targets.clear();
		for (BlockConnector connector : block.getPostConnectors()) {
			targets.add(connector.getTarget());
		}

		assertTrue(targets.contains(Activity.getActivity("A")));
		assertTrue(targets.contains(Activity.getActivity("B")));
		assertTrue(targets.contains(Activity.getActivity("C")));
		assertTrue(targets.contains(Activity.getActivity("D")));

		assertTrue(function.getPostset().size() == 1);

		assertTrue(postXor.getPostset().size() == 2);

		EPCNode secondAnd = null;
		for (EPCEdge edge : postXor.getPostset()) {
			if (!edge.getEnd().equals(postAnd)) {
				secondAnd = edge.getEnd();
				break;
			}
		}

		assertTrue(secondAnd instanceof EPCAndConnector);
		assertTrue(secondAnd.getPostset().size() == 2);

		for (EPCEdge edge : postAnd.getPostset()) {
			EPCNode node = edge.getEnd();
			assertTrue(node instanceof EPCXorConnector);
			assertTrue(node.getPostset().size() == 1);
			assertTrue(node.getPostset().firstElement().getEnd() instanceof BlockConnector);
		}

		assertTrue(block.getPostConnectors().size() == 4);

		for (BlockConnector connector : block.getPostConnectors()) {
			if (connector.getTarget().equals(Activity.getActivity("A"))) {
				assertTrue(connector.getPreset().size() == 1);
			} else if (connector.getTarget().equals(Activity.getActivity("B"))) {
				assertTrue(connector.getPreset().size() == 1);
			} else if (connector.getTarget().equals(Activity.getActivity("C"))) {
				assertTrue(connector.getPreset().size() == 1);
				EPCNode xor = connector.getPreset().firstElement().getStart();
				assertTrue(xor instanceof EPCXorConnector);
				assertTrue(xor.getPreset().size() == 2);
			} else if (connector.getTarget().equals(Activity.getActivity("D"))) {
				assertTrue(connector.getPreset().size() == 1);
			} else {
				fail("Unexpected connector: " + connector);
			}
		}
	}

	@Test
	public void testAddPreConnectorString() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		block.addPreConnector("TestAction");

		assertTrue(block.getPreConnectors().size() == 1);
		assertTrue(block.getPreConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("TestAction")));
	}

	@Test
	public void testAddPreConnectorActivity() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		Activity activity = Activity.getActivity("TestActivity");

		block.addPreConnector(activity);

		assertTrue(block.getPreConnectors().size() == 1);
		assertTrue(block.getPreConnectors().firstElement().getTarget()
				.equals(activity));
	}

	@Test
	public void testAddPostConnectorString() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		block.addPostConnector("TestAction");

		assertTrue(block.getPostConnectors().size() == 1);
		assertTrue(block.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("TestAction")));
	}

	@Test
	public void testAddPostConnectorActivity() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		Activity activity = Activity.getActivity("TestActivity");

		block.addPostConnector(activity);

		assertTrue(block.getPostConnectors().size() == 1);
		assertTrue(block.getPostConnectors().firstElement().getTarget()
				.equals(activity));
	}

	@Test
	public void testRemovePostConnector() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		Activity activity = Activity.getActivity("TestActivity");

		block.addPostConnector(activity);

		assertTrue(block.getPostConnectors().size() == 1);
		assertTrue(block.getPostConnectors().firstElement().getTarget()
				.equals(activity));

		block.removePostConnector(block.getPostConnectors().firstElement());

		assertTrue(block.getPostConnectors().isEmpty());
	}

	@Test
	public void testRemovePreConnector() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		Activity activity = Activity.getActivity("TestActivity");

		block.addPreConnector(activity);

		assertTrue(block.getPreConnectors().size() == 1);
		assertTrue(block.getPreConnectors().firstElement().getTarget()
				.equals(activity));

		block.removePreConnector(block.getPreConnectors().firstElement());

		assertTrue(block.getPreConnectors().isEmpty());
	}

	@Test
	public void testGetPostConnectors() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		assertTrue(block.getPostConnectors().isEmpty());

		Activity activity = Activity.getActivity("TestActivity");

		block.addPostConnector(activity);

		assertTrue(block.getPostConnectors().size() == 1);
		assertTrue(block.getPostConnectors().firstElement().getTarget()
				.equals(activity));
	}

	@Test
	public void testGetPreConnectors() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		assertTrue(block.getPreConnectors().isEmpty());

		Activity activity = Activity.getActivity("TestActivity");

		block.addPreConnector(activity);

		assertTrue(block.getPreConnectors().size() == 1);
		assertTrue(block.getPreConnectors().firstElement().getTarget()
				.equals(activity));
	}

	@Test
	public void testGetActivity() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		assertTrue(block.getActivity().equals(blockActivity));
	}

	@Test
	public void testAddToEpc() {
		BuildingBlock block = new BuildingBlock(blockActivity);

		block.addPreset(activityset1);
		block.addPreset(activityset2);

		assertTrue(block.getPreConnectors().size() == 4);

		block.addPostset(activityset1);

		assertTrue(block.getPostConnectors().size() == 3);

		EPC epc = new EPC();

		assertTrue(epc.getFunctions().isEmpty());
		assertTrue(epc.getXorConnectors().isEmpty());
		assertTrue(epc.getAndConnectors().isEmpty());
		assertTrue(epc.getLooseEnds().isEmpty());

		block.addToEpc(epc);

		assertTrue(epc.getFunctions().size() + " != 1", epc.getFunctions()
				.size() == 1);
		assertTrue(epc.getXorConnectors().size() + " != 9", epc
				.getXorConnectors().size() == 9);
		assertTrue(epc.getAndConnectors().size() == 3);
		assertTrue(epc.getLooseEnds().size() == 7);

		assertTrue(block.getFunctions().size() == 1);

		EPCFunction epcFunction = epc.getFunctions().firstElement();

		// test preset

		assertTrue(epcFunction.getPreset().size() == 1);

		EPCNode preXor = epcFunction.getPreset().firstElement().getStart();
		assertTrue(preXor instanceof EPCXorConnector);
		assertTrue(preXor.getPreset().size() == 2);

		EPCNode preAnd1 = preXor.getPreset().get(0).getStart();
		EPCNode preAnd2 = preXor.getPreset().get(1).getStart();

		if (preAnd1.getPreset().size() == 2) {
			preAnd1 = preXor.getPreset().get(1).getStart();
			preAnd2 = preXor.getPreset().get(0).getStart();
		}

		Vector<Activity> activities = new Vector<Activity>();

		assertTrue(preAnd1.getPreset().size() == 3);
		for (EPCEdge edge : preAnd1.getPreset()) {
			EPCNode xor = edge.getStart();
			assertTrue(xor instanceof EPCXorConnector);
			assertTrue(xor.getPreset().size() == 1);

			EPCNode looseEnd = xor.getPreset().firstElement().getStart();
			assertTrue(looseEnd instanceof LooseEnd);
			assertTrue(looseEnd.getPreset().isEmpty());

			activities.add(((LooseEnd) looseEnd).getEnd());
		}

		assertTrue(activities.size() == 3);
		assertTrue(activities.contains(Activity.getActivity("A")));
		assertTrue(activities.contains(Activity.getActivity("B")));
		assertTrue(activities.contains(Activity.getActivity("C")));

		activities.clear();

		assertTrue(preAnd2.getPreset().size() == 2);
		for (EPCEdge edge : preAnd2.getPreset()) {
			EPCNode xor = edge.getStart();
			assertTrue(xor instanceof EPCXorConnector);
			assertTrue(xor.getPreset().size() == 1);

			EPCNode looseEnd = xor.getPreset().firstElement().getStart();
			assertTrue(looseEnd instanceof LooseEnd);
			assertTrue(looseEnd.getPreset().isEmpty());

			activities.add(((LooseEnd) looseEnd).getEnd());
		}

		assertTrue(activities.size() == 2);
		assertTrue(activities.contains(Activity.getActivity("C")));
		assertTrue(activities.contains(Activity.getActivity("D")));

		// test postset

		assertTrue(epcFunction.getPostset().size() == 1);

		EPCNode postXor = epcFunction.getPostset().firstElement().getEnd();
		assertTrue(postXor instanceof EPCXorConnector);
		assertTrue(postXor.getPreset().size() == 1);

		EPCNode postAnd = postXor.getPostset().firstElement().getEnd();

		assertTrue(postAnd instanceof EPCAndConnector);

		activities.clear();

		assertTrue(postAnd.getPostset().size() == 3);
		for (EPCEdge edge : postAnd.getPostset()) {

			EPCNode xor = edge.getEnd();

			assertTrue(xor instanceof EPCXorConnector);
			assertTrue(xor.getPostset().size() == 1);

			EPCNode looseEnd = xor.getPostset().firstElement().getEnd();

			assertTrue(looseEnd instanceof LooseEnd);
			assertTrue(looseEnd.getPostset().isEmpty());

			activities.add(((LooseEnd) looseEnd).getEnd());
		}

		assertTrue(activities.size() == 3);
		assertTrue(activities.contains(Activity.getActivity("A")));
		assertTrue(activities.contains(Activity.getActivity("B")));
		assertTrue(activities.contains(Activity.getActivity("C")));
	}
}