package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.lpo;

import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Vertex;

public class TestLpo {

	@Test
	public void testLpo() {
		Lpo lpo = new Lpo("testLpo");
		assertTrue(lpo.getName().equals("testLpo"));

		assertTrue(lpo.getActivities().size() == 2);
		assertTrue(lpo.getActivities().contains(
				Activity.getActivity(Lpo.START_LABEL)));
		assertTrue(lpo.getActivities().contains(
				Activity.getActivity(Lpo.FINAL_LABEL)));

		assertTrue(lpo.getAllVertices().size() == 2);
		assertTrue(lpo.getAllVertices().contains(lpo.getStartVertex()));
		assertTrue(lpo.getAllVertices().contains(lpo.getFinalVertex()));

	}

	@Test
	public void testFinishConstruction() {
		Lpo lpo = new Lpo("testLpo2");

		Vertex ta = lpo.addVertex("TestA");
		Vertex tb = lpo.addVertex("TestB");

		lpo.connectVertices(ta, tb);

		lpo.finishConstruction();

		Vertex node = lpo.getStartVertex();

		assertTrue(node.getPreset().isEmpty());
		assertTrue(node.getSkeletonPreset().isEmpty());
		assertTrue(node.getSkeletonPostset().size() == 1);
		assertTrue(node.getPostset().size() == 3);
		assertTrue(node.getActivity().getLabel().equals(Lpo.START_LABEL));

		node = node.getSkeletonPostset().firstElement().getEnd();

		assertTrue(node.getPreset().size() == 1);
		assertTrue(node.getSkeletonPreset().size() == 1);
		assertTrue(node.getSkeletonPostset().size() == 1);
		assertTrue(node.getPostset().size() == 2);
		assertTrue(node.getActivity().getLabel().equals("TestA"));

		node = node.getSkeletonPostset().firstElement().getEnd();

		assertTrue(node.getPreset().size() == 2);
		assertTrue(node.getSkeletonPreset().size() == 1);
		assertTrue(node.getSkeletonPostset().size() == 1);
		assertTrue(node.getPostset().size() == 1);
		assertTrue(node.getActivity().getLabel().equals("TestB"));

		node = node.getSkeletonPostset().firstElement().getEnd();

		assertTrue(node.getPreset().size() == 3);
		assertTrue(node.getSkeletonPreset().size() == 1);
		assertTrue(node.getSkeletonPostset().isEmpty());
		assertTrue(node.getPostset().isEmpty());
		assertTrue(node.getActivity().getLabel().equals(Lpo.FINAL_LABEL));
	}

	@Test
	public void testGetActivities() {
		Lpo lpo = new Lpo("testLpo3");
		lpo.addVertex("TestC");

		assertTrue(lpo.getActivities().size() == 3);
		assertTrue(lpo.getActivities().contains(Activity.getActivity("TestC")));
		assertTrue(lpo.getActivities().contains(
				Activity.getActivity(Lpo.START_LABEL)));
		assertTrue(lpo.getActivities().contains(
				Activity.getActivity(Lpo.FINAL_LABEL)));
	}

	@Test
	public void testGetVertices() {
		Lpo lpo = new Lpo("testLpo4");
		Vertex d1 = lpo.addVertex("TestD");
		Vertex d2 = lpo.addVertex("TestD");

		assertTrue(lpo.getVertices("TestD").size() + " != 2",
				lpo.getVertices("TestD").size() == 2);
		assertTrue(lpo.getVertices("TestD").contains(d1));
		assertTrue(lpo.getVertices("TestD").contains(d2));

		assertTrue(lpo.getVertices(Lpo.START_LABEL).size() == 1);
		assertTrue(lpo.getVertices(Lpo.START_LABEL).contains(
				lpo.getStartVertex()));

		assertTrue(lpo.getVertices(Lpo.FINAL_LABEL).size() == 1);
		assertTrue(lpo.getVertices(Lpo.FINAL_LABEL).contains(
				lpo.getFinalVertex()));
	}

	@Test
	public void testGetAllVertices() {
		Lpo lpo = new Lpo("testLpo5");
		Vertex d1 = lpo.addVertex("TestE");
		Vertex d2 = lpo.addVertex("TestE");

		assertTrue(lpo.getAllVertices().size() + " != 4", lpo.getAllVertices()
				.size() == 4);
		assertTrue(lpo.getAllVertices().contains(d1));
		assertTrue(lpo.getAllVertices().contains(d2));
		assertTrue(lpo.getAllVertices().contains(lpo.getStartVertex()));
		assertTrue(lpo.getAllVertices().contains(lpo.getFinalVertex()));

	}

	@Test
	public void testDisconnectVertices() {
		Lpo lpo = new Lpo("testLpo6");

		Vertex g = lpo.addVertex("TestG");
		Vertex h = lpo.addVertex("TestH");

		lpo.connectVertices(g, h);

		assertTrue(g.getPostset().firstElement().getEnd().equals(h));
		assertTrue(h.getPreset().firstElement().getStart().equals(g));

		lpo.disconnectVertices(g, h);

		assertTrue(g.getPostset().size() + " != 0", g.getPostset().isEmpty());
		assertTrue(h.getPreset().isEmpty());

		Vertex i1 = lpo.addVertex("TestI");
		Vertex i2 = lpo.addVertex("TestI");

		lpo.connectVertices(i1, i2);

		assertTrue(i1.getPostset().firstElement().getEnd().equals(i2));
		assertTrue(i2.getPreset().firstElement().getStart().equals(i1));

		lpo.disconnectVertices(i1, i2);

		assertTrue(i1.getPostset().size() + " != 0", i1.getPostset().isEmpty());
		assertTrue(i2.getPreset().isEmpty());
	}

	@Test
	public void testConnectVertices() {
		Lpo lpo = new Lpo("testLpo7");

		Vertex f1 = lpo.addVertex("TestF");
		Vertex f2 = lpo.addVertex("TestF");

		lpo.connectVertices(f1, f2);

		assertTrue(f1.getPostset().firstElement().getEnd().equals(f2));
		assertTrue(f2.getPreset().firstElement().getStart().equals(f1));
	}

	@Test
	public void testRemoveVertex() {
		Lpo lpo = new Lpo("testLpo8");

		Vertex j1 = lpo.addVertex("TestJ");
		Vertex j2 = lpo.addVertex("TestJ");

		lpo.connectVertices(j1, j2);
		lpo.removeVertex(j2);

		assertTrue(!lpo.getAllVertices().contains(j2));
		assertTrue(j1.getPostset().isEmpty());

		assertNull(j2.getLpo());
	}

	@Test
	public void testAddVertexString() {
		Lpo lpo = new Lpo("testLpo9");

		Vertex k = lpo.addVertex("TestK");

		assertTrue(lpo.getAllVertices().contains(k));
		assertTrue(lpo.getActivities().contains(Activity.getActivity("TestK")));
	}

	@Test
	public void testAddVertexActivity() {
		Lpo lpo = new Lpo("testLpo10");

		Activity al = Activity.getActivity("TestL");
		Vertex l = lpo.addVertex(al);

		assertTrue(lpo.getAllVertices().contains(l));
		assertTrue(lpo.getActivities().contains(al));
	}

	@Test
	public void testAddVertexVertex() {
		Lpo lpo = new Lpo("testLpo10");

		Activity al = Activity.getActivity("TestL");
		Vertex l = new Vertex(al);
		lpo.addVertex(l);

		assertTrue(l.getLpo().equals(lpo));
		assertTrue(lpo.getAllVertices().contains(l));
		assertTrue(lpo.getActivities().contains(al));
	}

	@Test
	public void testGetStartVertex() {
		Lpo lpo = new Lpo("testLpo11");

		assertTrue(lpo.getStartVertex() != null);
		assertTrue(lpo.getStartVertex().getActivity().getLabel()
				.equals(Lpo.START_LABEL));
	}

	@Test
	public void testGetFinalVertex() {
		Lpo lpo = new Lpo("testLpo12");

		assertTrue(lpo.getFinalVertex() != null);
		assertTrue(lpo.getFinalVertex().getActivity().getLabel()
				.equals(Lpo.FINAL_LABEL));
	}

	@Test
	public void testGetName() {
		Lpo lpo = new Lpo("testLpo13");

		assertTrue(lpo.getName().equals("testLpo13"));
	}

}
