package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.lpo;

import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.LpoEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Vertex;

public class TestLpoEdge {

	private Lpo lpo;
	private Vertex v1;
	private Vertex v2;

	@Before
	public void setUp() throws Exception {
		lpo = new Lpo("testLpo");
		v1 = lpo.addVertex("v1");
		v2 = lpo.addVertex("v2");
	}

	@Test
	public void testLpoEdgeVertexVertex() {
		LpoEdge edge = new LpoEdge(v1, v2);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));
	}

	@Test
	public void testLpoEdgeVertexVertexBoolean() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(edge.isModeledEdge());
		assertTrue(!edge.isSkeletonEdge());
		assertTrue(!edge.isFoldingEdge());
	}

	@Test
	public void testIsSkeletonEdge() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(edge.isModeledEdge());
		assertTrue(!edge.isSkeletonEdge());
		assertTrue(!edge.isFoldingEdge());

		edge.setSkeletonEdge(true);

		assertTrue(edge.isSkeletonEdge());
	}

	@Test
	public void testSetSkeletonEdge() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(edge.isModeledEdge());
		assertTrue(!edge.isSkeletonEdge());
		assertTrue(!edge.isFoldingEdge());

		edge.setSkeletonEdge(true);

		assertTrue(edge.isSkeletonEdge());
	}

	@Test
	public void testIsModeledEdge() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(!edge.isSkeletonEdge());
	}

	@Test
	public void testSetModeledEdge() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(edge.isModeledEdge());
		assertTrue(!edge.isSkeletonEdge());
		assertTrue(!edge.isFoldingEdge());

		edge.setModeledEdge(false);

		assertTrue(!edge.isModeledEdge());
	}

	@Test
	public void testIsFoldingEdge() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(!edge.isFoldingEdge());
	}

	@Test
	public void testSetFoldingEdge() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(v2));

		assertTrue(edge.isModeledEdge());
		assertTrue(!edge.isSkeletonEdge());
		assertTrue(!edge.isFoldingEdge());

		edge.setFoldingEdge(true);

		assertTrue(edge.isFoldingEdge());
	}

	@Test
	public void testGetStart() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getStart().equals(v1));
	}

	@Test
	public void testGetEnd() {
		LpoEdge edge = new LpoEdge(v1, v2, true);

		assertTrue(edge.getEnd().equals(v2));
	}

	@Test
	public void testEqualsObject() {
		LpoEdge edge1 = new LpoEdge(v1, v2, true);
		LpoEdge edge2 = new LpoEdge(v1, v2, false);
		LpoEdge edge3 = new LpoEdge(v2, v1, true);

		assertTrue(edge1.equals(edge2));
		assertTrue(!edge1.equals(edge3));
	}

}
