package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.datastructure.lpo;

import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.LpoEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Vertex;

public class TestVertex {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory.getLogger(TestVertex.class);

	private Lpo lpo;
	private Lpo lpo2;
	private Vertex v1;
	private Vertex v2;

	@Before
	public void setUp() throws Exception {
		lpo = new Lpo("testLpo");
		lpo2 = new Lpo("testLpo2");
		v1 = lpo.addVertex("v1");

		v2 = lpo.addVertex("v2");

		lpo2.addVertex(v1);
		lpo2.addVertex(v2);
		lpo2.connectVertices(v1, v2);

		lpo2.finishConstruction();
	}

	@Test
	public void testVertex() {
		Vertex vertex = new Vertex(Activity.getActivity("test"));

		assertTrue(vertex.getActivity().getLabel().equals("test"));
		assertTrue(vertex.getLpo() == null);
		assertTrue(vertex.getPostActivities().isEmpty());
		assertTrue(vertex.getPostset().isEmpty());
		assertTrue(vertex.getPostVertices().isEmpty());
		assertTrue(vertex.getPreActivities().isEmpty());
		assertTrue(vertex.getPredecessorSet().isEmpty());
		assertTrue(vertex.getPreset().isEmpty());
		assertTrue(vertex.getPreActivities().isEmpty());
		assertTrue(vertex.getSkeletonPostset().isEmpty());
		assertTrue(vertex.getSkeletonPreset().isEmpty());
		assertTrue(vertex.getSuccessorSet().isEmpty());
	}

	@Test
	public void testSetLpo() {
		Vertex vertex = new Vertex(Activity.getActivity("test"));

		vertex.setLpo(lpo);
		assertTrue(vertex.getLpo().equals(lpo));

		vertex.setLpo(null);
		assertTrue(vertex.getLpo() == null);

		vertex.setLpo(lpo2);
		assertTrue(vertex.getLpo().equals(lpo2));
	}

	@Test
	public void testGetLpo() {
		Vertex vertex = new Vertex(Activity.getActivity("test"));

		vertex.setLpo(lpo);
		assertTrue(vertex.getLpo().equals(lpo));
	}

	@Test
	public void testGetActivity() {
		Vertex vertex = new Vertex(Activity.getActivity("test"));

		assertTrue(vertex.getActivity().equals(Activity.getActivity("test")));
	}

	@Test
	public void testGetPreset() {
		Vertex vertex = lpo2.addVertex(Activity.getActivity("test"));
		assertTrue(vertex.getPreset().isEmpty());

		lpo2.connectVertices(v1, vertex);

		assertTrue(vertex.getPreset().size() == 1);
		assertTrue(vertex.getPreset().firstElement().getStart().equals(v1));
	}

	@Test
	public void testGetPostset() {
		Vertex vertex = lpo2.addVertex(Activity.getActivity("test"));

		assertTrue(vertex.getPostset().isEmpty());

		lpo2.connectVertices(vertex, v2);

		assertTrue(vertex.getPostset().size() == 1);
		assertTrue(vertex.getPostset().firstElement().getEnd().equals(v2));
	}

	@Test
	public void testGetSkeletonPreset() {
		assertTrue(v1.getSkeletonPreset().size() == 1);
		assertTrue(v1.getSkeletonPreset().firstElement().getStart()
				.getActivity().getLabel().equals(Lpo.START_LABEL));
	}

	@Test
	public void testGetPredecessorSet() {
		LOG.debug("v1.getPredecessorSet: " + v1.getPredecessorSet());
		assertTrue(v1.getPredecessorSet().size() + " != 1", v1
				.getPredecessorSet().size() == 1);
		assertTrue(v1.getPredecessorSet().firstElement()
				.equals(Activity.getActivity(Lpo.START_LABEL)));
	}

	@Test
	public void testGetPreActivities() {
		assertTrue(v2.getPreActivities().size() == 2);
	}

	@Test
	public void testGetPreVertices() {
		assertTrue(v2.getPreVertices().size() == 2);
		assertTrue(v2.getPreVertices().contains(v1));
		assertTrue(v2.getPreVertices().contains(lpo2.getStartVertex()));

	}

	@Test
	public void testGetPostVertices() {
		assertTrue(v1.getPostVertices().size() == 2);
		assertTrue(v1.getPostVertices().contains(v2));
		assertTrue(v1.getPostVertices().contains(lpo2.getFinalVertex()));
	}

	@Test
	public void testGetSkeletonPostset() {
		assertTrue(v1.getSkeletonPostset().size() == 1);
		assertTrue(v1.getSkeletonPostset().firstElement().getEnd().equals(v2));
	}

	@Test
	public void testGetSuccessorSet() {
		assertTrue(v1.getSuccessorSet().size() == 1);
		assertTrue(v1.getSuccessorSet().firstElement().equals(v2.getActivity()));
	}

	@Test
	public void testGetPostActivities() {
		assertTrue(v1.getPostActivities().size() == 2);
		assertTrue(v1.getPostActivities().contains(v2.getActivity()));
		assertTrue(v1.getPostActivities().contains(
				lpo.getFinalVertex().getActivity()));
	}

	@Test
	public void testAddVertexToPreset() {
		Vertex vertex = lpo2.addVertex("test");
		LpoEdge edge = v1.addVertexToPreset(vertex);

		assertTrue(edge.getStart().equals(vertex));
		assertTrue(edge.getEnd().equals(v1));

		assertTrue(vertex.getPostset().contains(edge));
		assertTrue(v1.getPreset().contains(edge));
	}

	@Test
	public void testRemoveVertexFromPreset() {
		Vertex vertex = lpo2.addVertex("test");
		LpoEdge edge = v1.addVertexToPreset(vertex);

		assertTrue(edge.getStart().equals(vertex));
		assertTrue(edge.getEnd().equals(v1));

		assertTrue(vertex.getPostset().contains(edge));
		assertTrue(v1.getPreset().contains(edge));

		v1.removeVertexFromPreset(vertex);

		assertTrue(!vertex.getPostset().contains(edge));
		assertTrue(!v1.getPreset().contains(edge));
	}

	@Test
	public void testAddVertexToPostset() {
		Vertex vertex = lpo2.addVertex("test");
		LpoEdge edge = v1.addVertexToPostset(vertex);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(vertex));

		assertTrue(v1.getPostset().contains(edge));
		assertTrue(vertex.getPreset().contains(edge));
	}

	@Test
	public void testAddEdgeToPostset() {
		Vertex vertex = lpo2.addVertex("test");

		LpoEdge edge = new LpoEdge(v1, v2);

		vertex.addEdgeToPostset(edge);

		assertTrue(vertex.getPostset().contains(edge));
	}

	@Test
	public void testAddEdgeToPreset() {
		Vertex vertex = lpo2.addVertex("test");

		LpoEdge edge = new LpoEdge(v1, v2);

		vertex.addEdgeToPreset(edge);

		assertTrue(vertex.getPreset().contains(edge));
	}

	@Test
	public void testAddTransitivPostVertex() {
		Lpo testLpo = new Lpo("testLpoTransitive");

		Vertex a = testLpo.addVertex("A");
		Vertex b = testLpo.addVertex("B");
		Vertex c = testLpo.addVertex("C");

		testLpo.connectVertices(a, b);
		testLpo.connectVertices(b, c);

		LpoEdge edge = a.addTransitivPostVertex(c);

		assertTrue(edge.getStart().equals(a));
		assertTrue(edge.getEnd().equals(c));
		assertTrue(!edge.isModeledEdge());
		assertTrue(!edge.isFoldingEdge());
		assertTrue(!edge.isSkeletonEdge());

		assertTrue(a.getPostset().contains(edge));
	}

	@Test
	public void testAddTransitivPreEdge() {
		Lpo testLpo = new Lpo("testLpoTransitive");

		Vertex a = testLpo.addVertex("A");
		Vertex b = testLpo.addVertex("B");
		Vertex c = testLpo.addVertex("C");

		testLpo.connectVertices(a, b);
		testLpo.connectVertices(b, c);

		LpoEdge edge = a.addTransitivPostVertex(c);

		c.addTransitivPreEdge(edge);

		assertTrue(c.getPreset().contains(edge));
	}

	@Test
	public void testRemoveVertexFromPostset() {
		Vertex vertex = lpo2.addVertex("test");
		LpoEdge edge = v1.addVertexToPostset(vertex);

		assertTrue(edge.getStart().equals(v1));
		assertTrue(edge.getEnd().equals(vertex));

		assertTrue(v1.getPostset().contains(edge));
		assertTrue(vertex.getPreset().contains(edge));

		v1.removeVertexFromPostset(vertex);

		assertTrue(!v1.getPostset().contains(edge));
		assertTrue(!vertex.getPreset().contains(edge));
	}

}
