package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.logic;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.HashMap;
import java.util.Vector;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPC;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCAndConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCFunction;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCNode;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.EPCXorConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks.BlockConnector;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.epc.buildingblocks.BuildingBlock;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Activity;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.logic.EpcFolding;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.run.TestPaperExample;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.test.TestLpoGenerator;

public class TestEpcFoldingAlgorithm {

	@Before
	public void setUp() throws Exception {
		TestPaperExample.prepareLogger();
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testAddToSpecification() {
		EpcFolding algo = new EpcFolding();

		Vector<Lpo> lpos = TestLpoGenerator.getPaperLpos();
		algo.addToSpecification(lpos);

		assertTrue(algo.getSpecification().size() == lpos.size());

		for (Lpo lpo : lpos) {
			assertTrue(algo.getSpecification().contains(lpo));
		}

	}

	@Test
	public void testRunAlgo() {
		EpcFolding algo = new EpcFolding();

		Vector<Lpo> lpos = TestLpoGenerator.getPaperLpos();
		algo.addToSpecification(lpos);

		assertNotNull(algo.runAlgo());
	}

	@Test
	public void testGetEpc() {
		EpcFolding algo = new EpcFolding();

		Vector<Lpo> lpos = TestLpoGenerator.getPaperLpos();
		algo.addToSpecification(lpos);

		HashMap<Activity, BuildingBlock> blocks = algo
				.generateBuildingBlocks(algo.getSpecification());

		assertNotNull(algo.mergeBlocks(blocks));
	}

	@Test
	public void testGenerateBuildingBlocks() {
		EpcFolding algo = new EpcFolding();

		Vector<Lpo> lpos = TestLpoGenerator.getPaperLpos();
		algo.addToSpecification(lpos);

		HashMap<Activity, BuildingBlock> blocks = algo.generateBuildingBlocks(algo.getSpecification());

		assertTrue(blocks.keySet().size() == 9);

		assertTrue(blocks.keySet().contains(
				Activity.getActivity(Lpo.START_LABEL)));
		assertTrue(blocks.keySet().contains(
				Activity.getActivity(Lpo.FINAL_LABEL)));
		assertTrue(blocks.keySet().contains(Activity.getActivity("A")));
		assertTrue(blocks.keySet().contains(Activity.getActivity("B")));
		assertTrue(blocks.keySet().contains(Activity.getActivity("C")));
		assertTrue(blocks.keySet().contains(Activity.getActivity("D")));
		assertTrue(blocks.keySet().contains(Activity.getActivity("E")));
		assertTrue(blocks.keySet().contains(Activity.getActivity("F")));
		assertTrue(blocks.keySet().contains(Activity.getActivity("G")));

		for (Activity activity : blocks.keySet()) {
			blocks.get(activity).getActivity().equals(activity);
		}

		// start

		BuildingBlock start = blocks.get(Activity.getActivity(Lpo.START_LABEL));
		assertTrue(start.getPreConnectors().size() == 0);
		assertTrue(start.getPostConnectors().size() == 1);
		assertTrue(start.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("A")));

		for (EPCNode node : start.getXorConnectors()) {
			System.err.println("XOR: pre: " + node.getPreset().size()
					+ ", post: " + node.getPostset().size());
		}

		assertTrue(start.getXorConnectors().size() + " != 0", start
				.getXorConnectors().size() == 0);
		assertTrue(start.getAndConnectors().size() == 0);
		assertTrue(start.getFunctions().size() == 1);

		// a
		BuildingBlock a = blocks.get(Activity.getActivity("A"));
		assertTrue(a.getPreConnectors().size() == 2);
		assertTrue(a.getPostConnectors().size() == 4);

		Vector<Activity> preAc = new Vector<Activity>();
		for (BlockConnector con : a.getPreConnectors()) {
			preAc.add(con.getTarget());
		}
		assertTrue(preAc.contains(Activity.getActivity(Lpo.START_LABEL)));
		assertTrue(preAc.contains(Activity.getActivity("G")));

		Vector<Activity> postAc = new Vector<Activity>();
		for (BlockConnector con : a.getPostConnectors()) {
			postAc.add(con.getTarget());
		}
		assertTrue(postAc.contains(Activity.getActivity("B")));
		assertTrue(postAc.contains(Activity.getActivity("D")));
		assertTrue(postAc.contains(Activity.getActivity("E")));
		assertTrue(postAc.contains(Activity.getActivity("G")));

		assertTrue(a.getXorConnectors().size() == 3);
		assertTrue(a.getAndConnectors().size() == 2);
		assertTrue(a.getFunctions().size() == 1);

		// b
		BuildingBlock b = blocks.get(Activity.getActivity("B"));
		assertTrue(b.getPreConnectors().size() == 1);
		assertTrue(b.getPostConnectors().size() == 1);

		assertTrue(b.getPreConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("A")));
		assertTrue(b.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("C")));

		assertTrue(b.getXorConnectors().size() == 0);
		assertTrue(b.getAndConnectors().size() == 0);
		assertTrue(b.getFunctions().size() == 1);

		// c
		BuildingBlock c = blocks.get(Activity.getActivity("C"));
		assertTrue(c.getPreConnectors().size() == 1);
		assertTrue(c.getPostConnectors().size() == 1);

		assertTrue(c.getPreConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("B")));
		assertTrue(c.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("F")));

		assertTrue(c.getXorConnectors().size() == 0);
		assertTrue(c.getAndConnectors().size() == 0);
		assertTrue(c.getFunctions().size() == 1);

		// d
		BuildingBlock d = blocks.get(Activity.getActivity("D"));
		assertTrue(d.getPreConnectors().size() == 1);
		assertTrue(d.getPostConnectors().size() == 1);

		assertTrue(d.getPreConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("A")));
		assertTrue(d.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("F")));

		assertTrue(d.getXorConnectors().size() == 0);
		assertTrue(d.getAndConnectors().size() == 0);
		assertTrue(d.getFunctions().size() == 1);

		// e
		BuildingBlock e = blocks.get(Activity.getActivity("E"));
		assertTrue(e.getPreConnectors().size() == 1);
		assertTrue(e.getPostConnectors().size() == 1);

		assertTrue(e.getPreConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("A")));
		assertTrue(e.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("F")));

		assertTrue(e.getXorConnectors().size() == 0);
		assertTrue(e.getAndConnectors().size() == 0);
		assertTrue(e.getFunctions().size() == 1);

		// g
		BuildingBlock g = blocks.get(Activity.getActivity("G"));
		assertTrue(g.getPreConnectors().size() == 1);
		assertTrue(g.getPostConnectors().size() == 2);

		assertTrue(g.getPreConnectors().firstElement().getTarget()
				.equals(Activity.getActivity("A")));

		postAc.clear();
		for (BlockConnector con : g.getPostConnectors()) {
			postAc.add(con.getTarget());
		}
		assertTrue(postAc.contains(Activity.getActivity(Lpo.FINAL_LABEL)));
		assertTrue(postAc.contains(Activity.getActivity("A")));

		assertTrue(g.getXorConnectors().size() == 1);
		assertTrue(g.getAndConnectors().size() == 0);
		assertTrue(g.getFunctions().size() == 1);

		// f
		BuildingBlock f = blocks.get(Activity.getActivity("F"));
		assertTrue(f.getPreConnectors().size() == 3);
		assertTrue(f.getPostConnectors().size() == 1);

		preAc.clear();
		for (BlockConnector con : f.getPreConnectors()) {
			preAc.add(con.getTarget());
		}
		assertTrue(preAc.contains(Activity.getActivity("C")));
		assertTrue(preAc.contains(Activity.getActivity("D")));
		assertTrue(preAc.contains(Activity.getActivity("E")));

		assertTrue(f.getPostConnectors().firstElement().getTarget()
				.equals(Activity.getActivity(Lpo.FINAL_LABEL)));

		EPCNode funcf = f.getFunctions().firstElement();
		assertTrue(funcf.getPreset().size() == 1);
		EPCNode xor = funcf.getPreset().firstElement().getStart();
		assertTrue(xor instanceof EPCXorConnector);
		assertTrue(xor.getPostset().size() == 1);
		assertTrue(xor.getPreset().size() + " != 2",
				xor.getPreset().size() == 2);

		assertTrue(f.getXorConnectors().size() == 2);
		assertTrue(f.getAndConnectors().size() == 2);
		assertTrue(f.getFunctions().size() == 1);

		// FI
		BuildingBlock fi = blocks.get(Activity.getActivity(Lpo.FINAL_LABEL));
		assertTrue(fi.getPreConnectors().size() == 2);
		assertTrue(fi.getPostConnectors().size() == 0);

		preAc.clear();
		for (BlockConnector con : fi.getPreConnectors()) {
			preAc.add(con.getTarget());
		}
		assertTrue(preAc.contains(Activity.getActivity("F")));
		assertTrue(preAc.contains(Activity.getActivity("G")));

		assertTrue(fi.getXorConnectors().size() == 1);
		assertTrue(fi.getAndConnectors().size() == 0);
		assertTrue(fi.getFunctions().size() == 1);
	}

	@Test
	public void testMergeBlocks() {
		EpcFolding algo = new EpcFolding();

		Vector<Lpo> lpos = TestLpoGenerator.getPaperLpos();
		algo.addToSpecification(lpos);

		HashMap<Activity, BuildingBlock> blocks = algo.generateBuildingBlocks(algo.getSpecification());

		EPC epc = algo.mergeBlocks(blocks);

		assertNotNull(epc);

		assertTrue(epc.getInitialNodes().size() + " != 1", epc
				.getInitialNodes().size() == 1);

		EPCNode start = epc.getInitialNodes().firstElement();

		assertTrue(start instanceof EPCFunction);
		assertTrue(start.getLabel().equals(Lpo.START_LABEL));
		assertTrue(start.getPostset().size() == 1);

		EPCNode xor1 = start.getPostset().firstElement().getEnd();

		assertTrue(xor1 instanceof EPCXorConnector);

		assertTrue(xor1.getPreset().size() + " != 2",
				xor1.getPreset().size() == 2);
		assertTrue(xor1.getPostset().size() == 1);

		EPCNode a = xor1.getPostset().firstElement().getEnd();

		assertTrue(a instanceof EPCFunction);
		assertTrue(a.getPreset().size() == 1);
		assertTrue(a.getPostset().size() == 1);

		EPCNode xor2 = a.getPostset().firstElement().getEnd();

		assertTrue(xor2 instanceof EPCXorConnector);
		assertTrue(xor2.getPreset().size() == 1);
		assertTrue(xor2.getPostset().size() == 3);

		EPCNode g = null;
		EPCNode and1 = null;
		EPCNode and2 = null;

		EPCFunction b = null;
		EPCFunction e = null;

		for (EPCEdge edge : xor2.getPostset()) {
			if (edge.getEnd() instanceof EPCFunction) {
				g = edge.getEnd();
			} else {
				if (edge.getEnd().getPostset().get(0).getEnd() instanceof EPCFunction) {
					EPCFunction func = (EPCFunction) edge.getEnd().getPostset()
							.get(0).getEnd();
					if (func.getLabel().equals("B")) {
						b = func;
						and1 = edge.getEnd();
					} else {
						e = func;
						and2 = edge.getEnd();
					}
				} else {
					EPCFunction func = (EPCFunction) edge.getEnd().getPostset()
							.get(1).getEnd();
					if (func.getLabel().equals("B")) {
						b = func;
						and1 = edge.getEnd();
					} else {
						e = func;
						and2 = edge.getEnd();
					}
				}
			}
		}

		assertTrue(g instanceof EPCFunction);
		assertTrue(g.getPreset().size() == 1);
		assertTrue(g.getPostset().size() == 1);

		EPCNode xor3 = g.getPostset().firstElement().getEnd();

		assertTrue(xor3 instanceof EPCXorConnector);
		assertTrue(xor3.getPreset().size() == 1);
		assertTrue(xor3.getPostset().size() == 2);

		EPCNode xor7 = null;

		if (xor3.getPostset().get(0).getEnd() == xor1) {
			xor7 = xor3.getPostset().get(1).getEnd();
		} else {
			xor7 = xor3.getPostset().get(0).getEnd();
		}

		assertTrue(xor7 instanceof EPCXorConnector);
		assertTrue(xor7.getPreset().size() == 2);
		assertTrue(xor7.getPostset().size() == 1);

		EPCNode fi = xor7.getPostset().firstElement().getEnd();

		assertTrue(fi instanceof EPCFunction);

		assertTrue(fi.getPreset().size() == 1);
		assertTrue(fi.getPostset().isEmpty());

		assertTrue(and1 instanceof EPCAndConnector);
		assertTrue(and1.getPreset().size() == 1);
		assertTrue(and1.getPostset().size() == 2);

		EPCNode xor4 = null;

		if (and1.getPostset().get(0).getEnd() instanceof EPCFunction) {
			xor4 = and1.getPostset().get(1).getEnd();
			assertTrue(and1.getPostset().get(0).getEnd() == b);
		} else {
			xor4 = and1.getPostset().get(0).getEnd();
			assertTrue(and1.getPostset().get(1).getEnd() == b);
		}

		assertTrue(xor4 instanceof EPCXorConnector);
		assertTrue(xor4.getPreset().size() == 2);
		assertTrue(xor4.getPostset().size() == 1);

		assertTrue(b instanceof EPCFunction);
		assertTrue(b.getPreset().size() == 1);
		assertTrue(b.getPostset().size() == 1);

		EPCNode c = b.getPostset().firstElement().getEnd();

		assertTrue(c instanceof EPCFunction);
		assertTrue(c.getPreset().size() == 1);
		assertTrue(c.getPostset().size() == 1);

		EPCNode and3 = c.getPostset().firstElement().getEnd();

		assertTrue(and3 instanceof EPCAndConnector);
		assertTrue(and3.getPreset().size() == 2);
		assertTrue(and3.getPostset().size() == 1);

		EPCNode d = xor4.getPostset().firstElement().getEnd();

		assertTrue(d instanceof EPCFunction);
		assertTrue(d.getPreset().size() == 1);
		assertTrue(d.getPostset().size() == 1);

		EPCNode xor5 = d.getPostset().firstElement().getEnd();

		assertTrue(xor5 instanceof EPCXorConnector);
		assertTrue(xor5.getPreset().size() == 1);
		assertTrue(xor5.getPostset().size() == 2);

		EPCNode and4 = null;

		if (xor5.getPostset().get(0).getEnd() == and3) {
			and4 = xor5.getPostset().get(1).getEnd();
		} else {
			assertTrue(xor5.getPostset().get(1).getEnd() == and3);
			and4 = xor5.getPostset().get(0).getEnd();
		}

		assertTrue(and2 instanceof EPCAndConnector);
		assertTrue(and2.getPreset().size() == 1);
		assertTrue(and2.getPostset().size() == 2);
		assertTrue(and2.getPreset().firstElement().getStart() == xor2);

		assertTrue(and2.getPostset().get(0).getEnd() == e
				|| and2.getPostset().get(0).getEnd() == xor4);
		assertTrue(and2.getPostset().get(1).getEnd() == e
				|| and2.getPostset().get(1).getEnd() == xor4);

		assertTrue(e instanceof EPCFunction);
		assertTrue(e.getPreset().size() == 1);
		assertTrue(e.getPostset().size() == 1);

		assertTrue(e.getPostset().firstElement().getEnd() == and4);

		assertTrue(and4 instanceof EPCAndConnector);
		assertTrue(and4.getPreset().size() == 2);
		assertTrue(and4.getPostset().size() == 1);

		EPCNode xor6 = and4.getPostset().firstElement().getEnd();

		assertTrue(xor6 instanceof EPCXorConnector);
		assertTrue(xor6.getPreset().size() == 2);
		assertTrue(xor6.getPostset().size() == 1);

		assertTrue(xor6.getPreset().get(0).getStart() == and3
				|| xor6.getPreset().get(0).getStart() == and4);
		assertTrue(xor6.getPreset().get(1).getStart() == and3
				|| xor6.getPreset().get(1).getStart() == and4);

		EPCNode f = xor6.getPostset().firstElement().getEnd();

		assertTrue(f instanceof EPCFunction);
		assertTrue(f.getPreset().size() == 1);
		assertTrue(f.getPostset().size() == 1);

		assertTrue(f.getPostset().firstElement().getEnd() == xor7);

	}
}
