package de.fernunihagen.viptool.test.algorithm.folding.epcoflpo.test;

import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Lpo;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.LpoEdge;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.datastructure.lpo.Vertex;
import de.fernunihagen.viptool.algorithm.folding.epcoflpo.test.TestLpoGenerator;

public class TestTestLpoGenerator {

	/**
	 * Logger for this class
	 */
	private static final Logger LOG = LoggerFactory
			.getLogger(TestTestLpoGenerator.class);

	@Before
	public void setUp() throws Exception {
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testGetFirstPaperLpo() {
		Lpo lpo = TestLpoGenerator.getFirstPaperLpo();

		Vertex start = lpo.getStartVertex();

		assertTrue(lpo.getFinalVertex().getPostset().isEmpty());

		assertTrue(start.getSkeletonPostset().size() == 1);
		assertTrue(start.getSkeletonPreset().isEmpty());

		for (LpoEdge edge : start.getPostset()) {
			LOG.debug("Edge from " + start.getActivity().getLabel() + " to "
					+ edge.getEnd().getActivity().getLabel());
		}

		assertTrue("Postset size is " + start.getPostset().size() + " != 6.",
				start.getPostset().size() == 6);

		Vertex a = start.getSkeletonPostset().firstElement().getEnd();
		assertTrue(a.getActivity().getLabel().equals("A"));
		assertTrue("Size of the skeleton preset of a is "
				+ a.getSkeletonPreset().size() + " != 1.", a
				.getSkeletonPreset().size() == 1);
		assertTrue(a.getSkeletonPostset().size() == 2);

		Vertex fi = lpo.getFinalVertex();

		assertTrue(fi.getSkeletonPreset().size() == 1);
		assertTrue(fi.getSkeletonPostset().isEmpty());

		for (LpoEdge edge : fi.getPreset()) {
			LOG.debug("Edge from " + edge.getStart().getActivity().getLabel()
					+ " to " + edge.getEnd().getActivity().getLabel());
		}

		assertTrue("Preset size is " + fi.getPreset().size() + " != 6.", fi
				.getPreset().size() == 6);

		Vertex f = fi.getSkeletonPreset().firstElement().getStart();

		assertTrue(f.getPreset().firstElement().getStart().getActivity()
				.getLabel().equals("C")
				|| f.getPreset().firstElement().getStart().getActivity()
						.getLabel().equals("D"));

		assertTrue("Size of the skeleton preset of F is "
				+ f.getSkeletonPreset().size() + " != 2.", f
				.getSkeletonPreset().size() == 2);
		assertTrue(f.getSkeletonPostset().size() == 1);
	}

	@Test
	public void testGetSecondPaperLpo() {
		Lpo lpo = TestLpoGenerator.getSecondPaperLpo();

		Vertex start = lpo.getStartVertex();

		assertTrue(lpo.getFinalVertex().getPostset().isEmpty());

		assertTrue(start.getSkeletonPostset().size() == 1);
		assertTrue(start.getSkeletonPreset().isEmpty());

		for (LpoEdge edge : start.getPostset()) {
			LOG.debug("Edge from " + start.getActivity().getLabel() + " to "
					+ edge.getEnd().getActivity().getLabel());
		}

		assertTrue("Postset size is " + start.getPostset().size() + " != 5.",
				start.getPostset().size() == 5);

		Vertex a = start.getSkeletonPostset().firstElement().getEnd();
		assertTrue(a.getActivity().getLabel().equals("A"));
		assertTrue("Size of the skeleton preset of a is "
				+ a.getSkeletonPreset().size() + " != 1.", a
				.getSkeletonPreset().size() == 1);
		assertTrue(a.getSkeletonPostset().size() == 2);

		Vertex fi = lpo.getFinalVertex();

		assertTrue(fi.getSkeletonPreset().size() == 1);
		assertTrue(fi.getSkeletonPostset().isEmpty());

		for (LpoEdge edge : fi.getPreset()) {
			LOG.debug("Edge from " + edge.getStart().getActivity().getLabel()
					+ " to " + edge.getEnd().getActivity().getLabel());
		}

		assertTrue("Preset size is " + fi.getPreset().size() + " != 5.", fi
				.getPreset().size() == 5);

		Vertex f = fi.getSkeletonPreset().firstElement().getStart();

		assertTrue(f.getPreset().firstElement().getStart().getActivity()
				.getLabel().equals("E")
				|| f.getPreset().firstElement().getStart().getActivity()
						.getLabel().equals("D"));

		assertTrue("Size of the skeleton preset of F is "
				+ f.getSkeletonPreset().size() + " != 2.", f
				.getSkeletonPreset().size() == 2);
		assertTrue(f.getSkeletonPostset().size() == 1);
	}

	@Test
	public void testGetThirdPaperLpo() {
		Lpo lpo = TestLpoGenerator.getThirdPaperLpo();

		Vertex current = lpo.getStartVertex();

		String[] labels = new String[] { "ST", "A", "G", "A", "G", "FI" };

		for (int i = 0; i < 6; i++) {

			assertTrue(current.getActivity().getLabel().equals(labels[i]));

			assertTrue(current.getPreset().size() == i);
			assertTrue(current.getPostset().size() == (5 - i));

			if (i == 0) {
				assertTrue(current.getSkeletonPreset().size() == 0);
			} else {
				assertTrue(current.getSkeletonPreset().size() == 1);
			}

			if (i == 5) {
				assertTrue(current.getSkeletonPostset().size() == 0);
			} else {
				assertTrue(current.getSkeletonPostset().size() == 1);
			}

			if (i != 5) {
				current = current.getSkeletonPostset().firstElement().getEnd();
			}
		}
	}
}
